(* TEST.PAS ****************************************************************)
(* Program przykladowy pokazujacy sposob wykorzystania driver'ow modulow   *)
(* serii LC-011 i LC-020 firmy AMBEX.                                      *)
(***************************************************************************)

program test;

uses crt, dos;

{$i ambex-lc.pas}

type
    buftype = array[1..320] of integer;		(* bufor na probki         *)
    buftypeptr = ^buftype;

var
    s_init:      	lc0_init;
    s_total:     	lc0_total;
    s_module:    	lc0_module;
    s_info:      	lc0_info;
    s_break:     	lc0_break;
    s_analog_in: 	lc0_analog_in;
    s_analog_out:	lc0_analog_out;
    s_leave:     	lc0_leave;
    s_interr:		lc0_interrupt;

    r: registers;			(* parametr dla funkcji intr       *)
    LCinterrupt: integer;		(* numer przerwania driver'a       *)
    modulenum: integer;			(* numer badanego modulu           *)
    koniec: integer;			(* parametr ustawiany w funkcji	   *)
					(* dla funkcji my_break            *)
	 bufnum: integer;		(* parametr ustawiany w funkcji	   *)
					(* dla pracy na dwa bufory         *)
	 buf1: pointer;		(* parametry dla funkcji cycletransmission *)
	 buf2: pointer;
	 sstat: integer;                     (* slowo komunikacyjne dla funkcji *)
					(* obslugi przerwania z modulu     *)

const
    BUFLEN	= 320;				(* dlugosc bufora          *)
    SAMPLES	= 20;				(* liczba probek           *)
    CHANNELS	= 8;				(* liczba kanalow          *)

(***************************************************************************)
(* Przeznaczenie:                                                          *)
(*   Wykonanie wlasnej obslugi Ctrl-Break.                                 *)
(* Sposob:                                                                 *)
(*   Przez wykonanie funkcji BREAK.                                        *)
(***************************************************************************)
procedure my_break(Flags, CS, IP, AX, BX, CX, DX, SI, DI,
		   DS, ES, BP: Word); interrupt;
begin

    koniec := 1;
    s_break.LC0_BMODE := LC0_BREAK_EXEC;
    r.dx := seg(s_break);
    r.di := ofs(s_break);
    intr(LCinterrupt, r);				(* BREAK    *)

end;

(***************************************************************************)
(* Przeznaczenie:                                                          *)
(*   Zamiana cyfry szesnastkowej na jej reprezentacje znakowa.             *)
(* Parametry:                                                              *)
(*   dig - cyfra do zamiany                                                *)
(* Wartosc:                                                                *)
(*   Reprezentacja znakowa cyfry.                                          *)
(***************************************************************************)
function hexdigit(dig: word) : char;
begin
    if dig < 10 then hexdigit := chr(ord('0') + dig)
    else             hexdigit := chr(ord('A') + dig - 10);
end;

(***************************************************************************)
(* Przeznaczenie:                                                          *)
(*   Wydrukowanie liczby calkowitej w postaci szesnastkowej.               *)
(* Parametry:                                                              *)
(*   val - wartosc do wydrukowania                                         *)
(***************************************************************************)
procedure writehex(val: word);
begin
    write(hexdigit((val and $F000) shr 12));
    write(hexdigit((val and $F00) shr 8));
    write(hexdigit((val and $F0) shr 4));
    write(hexdigit(val and $F));
end;

(***************************************************************************)
(* Przeznaczenie:                                                          *)
(*   Inicjalizacja zmiennych.                                              *)
(***************************************************************************)
procedure initprogram;
begin
	 koniec := 0;
	 bufnum := 1;
	 s_init.LC0_CODE		:= MODULE_INIT;
    s_total.LC0_CODE		:= GET_TOTAL_CONFIGURATION;
    s_module.LC0_CODE		:= GET_MODULE_CONFIGURATION;
    s_info.LC0_CODE		:= GET_INFO;
    s_break.LC0_CODE		:= BREAK;
    s_analog_in.LC0_CODE	:= ANALOG_INPUT;
    s_analog_out.LC0_CODE	:= ANALOG_OUTPUT;
    s_leave.LC0_CODE		:= LEAVE_DRIVER;
    s_interr.LC0_CODE		:= INTERRUPT_SERVICE;
end;

(***************************************************************************)
(* Przeznaczenie:                                                          *)
(*   Sprawdzenie obecnosci drivera.                                        *)
(* Sposob:                                                                 *)
(*   Przez probe otwarcia urzadzenia o nazwie okreslonej przez driver.     *)
(* Parametry:                                                              *)
(*   name - nazwa urzadzenia                                               *)
(* Wartosc:                                                                *)
(*   TRUE  - driver jest zainstalowany                                     *)
(*   FALSE - driver nie jest zainstalowany                                 *)
(***************************************************************************)
function driverinstalled(name: string) : boolean;
var
    hd: text;
begin
{$I-}
    assign(hd, name);
    reset(hd);
    if IOResult <> 0 then
	driverinstalled := false
    else
	begin
	    close(hd);
	    driverinstalled := true
	end;
{$I+}
end;

(***************************************************************************)
(* Przeznaczenie:                                                          *)
(*   Rozpoznanie, ktory driver (a co za tym idzie ktory modul) jest        *)
(*   zainstalowany.                                                        *)
(* Sposob:                                                                 *)
(*   Przez sprawdzenie zainstalowania kazdego z potencjalnych driver'ow.   *)
(* Uwagi:                                                                  *)
(*   Funkcja dodatkowo wyswietla komunikat o testowanym module i podstawia *)
(*   wlasciwa wartosc na zmienna LCinterrupt (wlasciwy numer przerwania    *)
(*   driver'a). Jezeli zaden driver nie jest zainstalowany to po           *)
(*   wyswietleniu wlasciwego komunikaru program konczy prace.              *)
(***************************************************************************)
procedure checkdrivers;
begin
    clrscr;
    gotoxy(1, 1);
    if driverinstalled('LC1116^^') then
	begin
	    writeln('Test modulu LC-011-1612');
	    LCinterrupt := LC011_16;
	end
    else if driverinstalled('LC2008^2') then
	begin
	    writeln('Test modulu LC-020-0812');
	    LCinterrupt := LC020_08_2;
	end
    else if driverinstalled('LC2032^^') then
	begin
	    writeln('Test modulu LC-020_3212');
	    LCinterrupt := LC020_32;
	end
    else
	begin
	    writeln('Driver nie jest zainstalowany!');
	    halt;
	end;
end;

(***************************************************************************)
(* Przeznaczenie:                                                          *)
(*   Rozpoznanie konfiguracji badanego modulu.                             *)
(* Sposob:                                                                 *)
(*   Przez wykorzystanie funkcji driver'a GET_TOTAL_CONFIGURATION,         *)
(*   GET_MODULE_CONFIGURATION, GET_INFO.                                   *)
(* Uwagi:                                                                  *)
(*   Funkcja nadaje wartosc zmiennej modulenum (numer badanego modulu)     *)
(*   wypelnia struktury total, module, info i inicjalizuje zainstalowane   *)
(*    moduly.                                                              *)
(***************************************************************************)
procedure askdriver;
label	lab1;
begin
    r.dx := seg(s_total);
    r.di := ofs(s_total);
    intr(LCinterrupt, r);			(* GET_TOTAL_CONFIGURATION *)

				(* inicjalizacja zainstalowanych modulow   *)
    s_init.LC0_IMODULE := s_total.LC0_TONF and $F;
    r.dx := seg(s_init);
    r.di := ofs(s_init);
    intr(LCinterrupt, r);				(* MODULE_INIT     *)

					(* sprawdzenie, ktory modul jest   *)
					(* zainstalowany: A, B, C czy D    *)
    for modulenum := 1 to 4 do
	if ((s_total.LC0_TONF and (1 shl (modulenum - 1))) <> 0) then
	    goto lab1;

lab1:
					(* spytanie o konfiguracje modulu  *)
    s_module.LC0_MMODULE := modulenum;
    r.dx := seg(s_module);
    r.di := ofs(s_module);
    intr(LCinterrupt, r);			(* GET_MODULE_CONFIGURATION*)

					(* spytanie o konfiguracje toru a/c*)
    s_info.LC0_GMODULE := modulenum;
    s_info.LC0_GTYPE   := LC0_AINPUT;
    s_info.LC0_GNUM    := 1;
    r.dx := seg(s_info);
    r.di := ofs(s_info);
    intr(LCinterrupt, r);				(* GET_INFO        *)
end;

(***************************************************************************)
(* Przeznaczenie:                                                          *)
(*   Wyswietlenie konfiguracji badanego modulu i jego toru a/c.            *)
(* Sposob:                                                                 *)
(*   Przez wykorzystanie informacji zawartych w strukturach module i info. *)
(***************************************************************************)
procedure displayconfiguration;
var
    i: integer;
begin
					(* konfiguracja modulu              *)
    writeln;
    writeln('---------- Konfiguracja modulu:');
    writeln;
    write('Adres modulu: ');
    writehex(s_module.LC0_MBASE1);
    writeln(' (hex)');
    writeln('Liczba przetwornikow a/c: ', s_module.LC0_MIAD);
    writeln('Liczba przetwornikow c/a: ', s_module.LC0_MIDA);
    writeln('Liczba portow cyfrowych wejsciowych: ', s_module.LC0_MIDI);
    writeln('Liczba portow cyfrowych wyjsciowych: ', s_module.LC0_MIDO);
    writeln('Czestotliwosc zegara modulu:  ',
	    (s_module.LC0_MCLOCK / 1000):4:2, 'MHz');

					(* konfiguracja toru a/c            *)
    writeln;
    writeln('---------- Konfiguracja toru a/c:');
    writeln;
    writeln('Liczba kanalow: ', s_info.LC0_GCHAN);
    writeln('Rozdzielczosc: ', s_info.LC0_GRES, ' bitow');
    writeln('Zakres napiec: ', (s_info.LC0_GMINV / 10):5:1, '..',
	    (s_info.LC0_GMAXV / 10):4:1, ' V');
    if s_info.LC0_GDMA = $FF then
	writeln('Tor a/c nie jest podlaczony do kanalow DMA')
    else
	writeln('Numer kanalu DMA podlaczonego do toru a/c: ',
							s_info.LC0_GDMA);
    writeln('Minimalne okresy probkowania [s]:');
    write('     ');
    for i := 1 to s_info.LC0_GCHAN do
	begin
	    write((s_info.LC0_GMINP[i] / 10):5:1);
	    if i < s_info.LC0_GCHAN then
		begin
		    write(', ');
		    if (i mod 8) = 0 then
			begin
			    writeln;	(* zlamanie linii co 8 wielkosci   *)
			    write('     ');
			end
		end
	end;
    writeln;
end;

(***************************************************************************)
(* Przeznaczenie:                                                          *)
(*   Zainstalowanie procedury obslugi przerwania generowanego przez        *)
(*   Ctrl-Break.                                                           *)
(* Sposob:                                                                 *)
(*   Przez wywolanie funkcji BREAK.                                        *)
(***************************************************************************)
procedure installbreak;
begin
    s_break.LC0_BMODE := LC0_BREAK_INST;
    s_break.LC0_BPROC := @my_break;	(* podlozona zostanie funkcja      *)
					(* my_break   			   *)
    r.dx := seg(s_break);
    r.di := ofs(s_break);
    intr(LCinterrupt, r);				(* BREAK           *)
end;

(***************************************************************************)
(* Przeznaczenie:                                                          *)
(*   Przerwa miedzy kolejnymi czesciami programu - oczekiwanie na reakcje  *)
(*   operatora.                                                            *)
(***************************************************************************)
procedure pressanykey;
var
    c: char;
begin
    writeln;
    writeln('Nacisnij dowolny klawisz . . .');
    c := readkey;
    clrscr;
    gotoxy(1, 1);
end;

(***************************************************************************)
(* Przeznaczenie:                                                          *)
(*   Wyswietlenie bufora z danymi pomiarowymi.                             *)
(* Parametry:                                                              *)
(*   buf - adres bufora                                                    *)
(*   c   - liczba kanalow                                                  *)
(*   s   - liczba probek na kanal                                          *)
(***************************************************************************)
procedure displaybuf(var buf: buftype; c, s: integer);
var
    i: integer;
    j: integer;
begin

    for i := 1 to s do
	begin
	    write(i:2, ': ');
	    for j := 1 to c do
		begin
		    writehex(buf[(i - 1) * c + j]);
		    write('  ');
		end;
	    writeln;
	end;
end;

(***************************************************************************)
(* Przeznaczenie:                                                          *)
(*   Wyswietlenie bufora z danymi pomiarowymi /praca asynchroniczna/.      *)
(* Parametry:                                                              *)
(*   buf - adres bufora                                                    *)
(*   c   - liczba kanalow                                                  *)
(*   s   - liczba probek na kanal                                          *)
(***************************************************************************)
procedure displaybuf1(var buf: buftypeptr; c, s: integer);
var
    i: integer;
    j: integer;
    qq: word;
begin
    gotoxy(1,5);
    for i := 1 to s do
	begin
	    write(i:2, ': ');
	    for j := 1 to c do
		begin
		    qq := word(buf^[ (i - 1) * c + j ]);
		    writehex(qq);
		    write('  ');
		end;
	    writeln;
	end;
end;

(***************************************************************************)
(* Przeznaczenie:                                                          *)
(*   Wykonanie obliczen w czasie pomiaru do drugiego bufora.               *)
(* Parametry:                                                              *)
(*   buf - adres bufora przetwarzanego                                     *)
(*   c   - liczba kanalow                                                  *)
(*   s   - liczba probek na kanal                                          *)
(*************************************************************2**************)
procedure performcalc(var buf: buftypeptr; c, s: integer);
type
    avgtype = array[1..8] of word;		(* bufor do obliczen         *)

var
	 i: integer;
    j: integer;
    qq: word;
    avrege: avgtype;

begin
	 for i := 1 to c do   		(* zerowanie wektora srednich *)
		avrege[i] := 0;

	 for i := 1 to s do   		(* liczenie sredniej w kanalach *)
		 for j := 1 to c do
			avrege[j] := avrege[j] + word(buf^[ (i - 1) * c + j ]);
	 for j := 1 to c do
		avrege[j] := avrege[j] DIV s;

	 gotoxy(1,4);
	 writeln('Praca z dwoma buforami');
	 gotoxy(1,7);
	 writeln('Srednie w poszczegolnych kanalach bufora do obliczen:');
	 writeln;
	 for j := 1 to c do
		begin
			 qq := avrege[j];
			 writehex(qq);
			 write('  ');
		end;
	 writeln;
	 writeln;
	 write('      Czekamy na zakoczenie pomiaru do drugiego bufora ...');
	 repeat
		;					(* oczekiwanie na koniec pomiaru *)
	 until ( koniec <> 0 );
end;

(***************************************************************************)
(* Przeznaczenie:                                                          *)
(*   Wypisanie komunikatow o bledzie / ostrzezeniu / dodatkowej informacji *)
(*   bledzie.                                                              *)
(* Parametry:                                                              *)
(*   status   - LC0_STATUS                                                 *)
(*   err_stat - LC0_ERR_STAT                                               *)
(***************************************************************************)
procedure drivererror(status, err_stat: shortint);
begin
    if status > 0 then
	writeln('--------- Ostrzezenie: ', status)
    else
	writeln('--------- Blad: ', status);
    if err_stat < 0 then
	writeln('--------- Informacje dodatkowe: ', err_stat);
end;

(***************************************************************************)
(* Przeznaczenie:                                                          *)
(*   Wykonanie transmisji blokowej.                                        *)
(* Sposob:                                                                 *)
(*   Przez wykonanie funkcji ANALOG_INPUT.                                 *)
(* Parametry:                                                              *)
(*   start - typ warunku startu                                            *)
(* Wartosc:                                                                *)
(* Uwagi:                                                                  *)
(*   Funkcja wywolujaca musi ustawic okres probkowania i parametry warunku *)
(*   startu.                                                               *)
(***************************************************************************)
procedure transmission(start: byte);
var
    buf: buftype;			(* bufor na probki                 *)
    c: char;
begin
    writeln('Pomiar blokowy, ', CHANNELS, ' kanalow, ', SAMPLES,
		 ' probek, okres probkowania [s]: ',
	    (s_analog_in.LC0_APER / 10):11:1, ' s:');
    case start of
	LC0_SIMMED:
	    writeln('Warunek startu: natychmiast');
	LC0_STIME:
	    writeln('Warunek startu: uplyw ', s_analog_in.LC0_ASTART.time,
		    ' sekund');
				(* pozostale warunki nie sa obslugiwane bo *)
				(* w programie nie sa wykorzystywane       *)
	end;
    s_analog_in.LC0_AMODULE := modulenum;
    s_analog_in.LC0_ANUM := 1;
    s_analog_in.LC0_AMODE := LC0_MOD_START or
			     LC0_MOD_NEW_PAR or
			     LC0_MOD_SYNCHR or
			     LC0_MOD_BLOCK;
				(* stop po zmierzeniu okreslonej liczby    *)
				(* probek                                  *)
    s_analog_in.LC0_ASTST := start + LC0_ZSAMPLES;
				(* praca wielokanalowa, CHANNELS kanalow   *)
    s_analog_in.LC0_ACHAN := CHANNELS;
    s_analog_in.LC0_AADDR := @buf;
    s_analog_in.LC0_ALEN := BUFLEN;	(* dlugosc bufora                  *)
    s_analog_in.LC0_ABMAR := 0;
    s_analog_in.LC0_AEMAR := 0;		(* oba marginesy zerowe            *)
					(* calkowita liczba probek         *)
    s_analog_in.LC0_ASTOP.samples := SAMPLES * CHANNELS;
    r.dx := seg(s_analog_in);
    r.di := ofs(s_analog_in);
    intr(LCinterrupt, r);				(* ANALOG_INPUT    *)
    if s_analog_in.LC0_STATUS <> LC0_OK then
	begin
	    if s_analog_in.LC0_STATUS = LC0_BROKEN then
	    	c := readkey;	(* oproznienie buf. klawiatury po Ctrl-Break*)
	    drivererror(s_analog_in.LC0_STATUS, s_analog_in.LC0_ERR_STAT)
	end
    else
	displaybuf(buf, CHANNELS, SAMPLES);
    pressanykey;
end;

(***************************************************************************)
(* Przeznaczenie:                                                          *)
(*   Wykonanie poprawnej transmisji blokowej i wyswietlenie zmierzonych    *)
(*   wartosci.                                                             *)
(* Sposob:                                                                 *)
(*   Przez wykonanie funkcji transmission.                                 *)
(***************************************************************************)
procedure blocktransmission;
begin
    writeln('---------- Poprawne przetwarzanie blokowe');
    writeln;
    s_analog_in.LC0_APER := s_info.LC0_GMINP[CHANNELS];
    transmission(LC0_SIMMED);
end;

(***************************************************************************)
(* Przeznaczenie:                                                          *)
(*   Wykonanie blednej transmisji blokowej.                                *)
(* Sposob:                                                                 *)
(*   Przez wykonanie funkcji ANALOG_INPUT z okresem probkowania mniejszym  *)
(*   niz minimalny wskazany przez driver.                                  *)
(***************************************************************************)
procedure failblocktransmission;
begin
    writeln('---------- Bledne przetwarzanie blokowe');
    writeln;
    s_analog_in.LC0_APER := s_info.LC0_GMINP[CHANNELS] - 1;
    transmission(LC0_SIMMED);
end;

(***************************************************************************)
(* Przeznaczenie:                                                          *)
(*   Wykonanie poprawnej transmisji blokowej ale przerwanej przez operatora*)
(*   (Ctrl-Break) w trakcie czekania na spelnienie warunku startu.         *)
(* Sposob:                                                                 *)
(*   Przez wykonanie funkcji ANALOG_INPUT z warunkiem startu LC0_STIME     *)
(*   (start po okreslonym czasie) i parametrem tego warunku - 1000s.       *)
(***************************************************************************)
procedure interruptedbefore;
begin
    writeln('---------- Przetwarzanie blokowe z oczekiwaniem 1000s');
    writeln('---------- Oczekiwanie nalezy przerwac Ctrl-Break');
    writeln;
    s_analog_in.LC0_APER := s_info.LC0_GMINP[CHANNELS];
    s_analog_in.LC0_ASTART.time := 1000;
    transmission(LC0_STIME);
end;

(***************************************************************************)
(* Przeznaczenie:                                                          *)
(*   Wykonanie poprawnej transmisji blokowej ale przerwanej przez operatora*)
(*   (Ctrl-Break) w trakcie pomiaru.                                       *)
(* Sposob:                                                                 *)
(*   Przez wykonanie funkcji ANALOG_INPUT z okresem probkowania 100s.      *)
(***************************************************************************)
procedure interruptedafter;
begin
    writeln('---------- Przetwarzanie blokowe z okresem probkowania 100s');
    writeln('---------- Pomiar nalezy przerwac Ctrl-Break');
    writeln;
    s_analog_in.LC0_APER := 1000000000;
    transmission(LC0_SIMMED);
end;

(***************************************************************************)
(* Przeznaczenie:                                                          *)
(*   Wykonanie pomiaru bloku probek za pomoca tranmsisji pojedynczej, przy *)
(*   czym caly blok ma byc zmierzony po 5s od startu.                      *)
(* Sposob:                                                                 *)
(*   Przez wykonanie funkcji ANALOG_INPUT.                                 *)
(* Parametry:                                                              *)
(* Wartosc:                                                                *)
(* Uwagi:                                                                  *)
(***************************************************************************)
procedure singletransmission;
var
    buf: buftype;			(* bufor na probki                  *)
    i: integer;
    j: integer;
begin
    writeln('---------- Przetwarzanie pojedyncze - po 5 sekundach');
    writeln;
    s_analog_in.LC0_AMODULE := modulenum;
    s_analog_in.LC0_ANUM := 1;
    s_analog_in.LC0_AMODE := LC0_MOD_START or
			     LC0_MOD_NEW_PAR or
			     LC0_MOD_SINGLE;
				(* praca wielokanalowa, CHANNELS kanalow   *)
    s_analog_in.LC0_ACHAN := CHANNELS;
    s_analog_in.LC0_AADDR := @buf;
    s_analog_in.LC0_ALEN := CHANNELS;	(* dlugosc bufora (tylko na jeden  *)
    					(* pomiar)                         *)
    s_analog_in.LC0_ASTST := LC0_STIME;
    s_analog_in.LC0_ASTART.time := 5;	(* start po 5 sekundach            *)
    r.dx := seg(s_analog_in);
    r.di := ofs(s_analog_in);
    intr(LCinterrupt, r);				(* ANALOG_INPUT    *)
    s_analog_in.LC0_ASTST := LC0_SIMMED;
    s_analog_in.LC0_AADDR := @buf[CHANNELS + 1];
    i := 3;
    r.dx := seg(s_analog_in);
    r.di := ofs(s_analog_in);
    intr(LCinterrupt, r);				(* ANALOG_INPUT    *)
					(* zgaszenie bitu LC0_MOD_NEW_PAR  *)
    s_analog_in.LC0_AMODE := s_analog_in.LC0_AMODE and (not LC0_MOD_NEW_PAR);
    for j := 1 to 18 do
	begin
	    s_analog_in.LC0_AADDR := @buf[(i - 1) * CHANNELS + 1];
	    i := i + 1;
	    r.dx := seg(s_analog_in);
	    r.di := ofs(s_analog_in);
	    intr(LCinterrupt, r);			(* ANALOG_INPUT    *)
	end;
    displaybuf(buf, CHANNELS, SAMPLES);
    pressanykey;
end;

(****************************************************************************)
(* Przeznaczenie:                                                           *)
(*   Obsluga przerwania przychodzacego z modulu po zmierzeniu bloku         *)
(*   probek - praca asynchroniczna z przerwaniem na koniec bloku            *)
(* Sposob:                                                                  *)
(* Parametry:                                                               *)
(* Wartosc:                                                                 *)
(* Uwagi:                                                                   *)
(****************************************************************************)
{$S-}
procedure a_interrupt_service;
var
	a, i : integer;
begin
	if(sstat AND LC0_IS_END_ADC <> 0) then
	  begin
	  r.dx := seg(s_analog_in);
	  r.di := ofs(s_analog_in);
	  intr(LCinterrupt, r);		(* ANALOG_INPUT             	    *)
					(* funkcja MUSI !! byc wywolana w   *)
					(* obsludze przerwania z modulu     *)
	  sound(500);
	  for i := 0 to 32000 do a := i;
	  nosound;
	  koniec := 1;
	  end;
end;

{$S+}
(***************************************************************************)
(* Przeznaczenie:                                                          *)
(*   Wykonanie asynchronicznej transmisji blokowej.                        *)
(* Sposob:                                                                 *)
(*   Przez wykonanie funkcji ANALOG_INPUT.                                 *)
(* Parametry:                                                              *)
(* Wartosc:                                                                *)
(* Uwagi:                                                                  *)
(*   Funkcja wywolujaca musi ustawic okres probkowania i parametry warunku *)
(*   startu.                                                               *)
(***************************************************************************)
const  BUFLEN1:word = $FFFF;
procedure asynchrtransmission;
var
	seg_, last: word;
	Buffer_to_free: pointer;
	bufptr: buftypeptr;

begin

	sstat := 0;
	GetMem(buf1, BUFLEN1);
	if(buf1 = nil) then
	begin
	  writeln('BLAD - brak pamieci');
	  Halt(0);
	end;
	bufptr := buftypeptr(buf1);

			(* wyinstalowanie obslugi przerwania z modulu      *)
	s_interr.LC0_SMODULE := 1;
	s_interr.LC0_SPROC := nil;
	s_interr.LC0_SSTAT := @sstat;
	r.dx := seg(s_interr);
	r.di := ofs(s_interr);
	r.ds := seg(sstat);
	intr(LCinterrupt, r);                 	(* INTERRUPT_SERVICE *)
			(* zainstalowanie obslugi przerwania z modulu      *)
	s_interr.LC0_SPROC := @a_interrupt_service;
	r.dx := seg(s_interr);
	r.di := ofs(s_interr);
	r.ds := seg(sstat);
	intr(LCinterrupt, r);			(* INTERRUPT_SERVICE *)

	FillChar(buf1^, BUFLEN, 0);
	s_analog_in.LC0_APER := 5000000;
	writeln
	(
	'Pomiar blokowy ,' ,CHANNELS, ' kanalow ',SAMPLES, ' probek, okres probkowania [s]: ', s_analog_in.LC0_APER DIV 10
	);
	s_analog_in.LC0_AMODULE := modulenum;
	s_analog_in.LC0_ANUM := 1;
	s_analog_in.LC0_AMODE := LC0_MOD_START OR
			      LC0_MOD_NEW_PAR OR
			      LC0_MOD_INTR OR
			      LC0_MOD_BLOCK;
			(* stop po zmierzeniu okreslonej liczby probek     *)
	s_analog_in.LC0_ASTST := LC0_ZSAMPLES;
				(* praca wielokanalowa, CHANNELS kanalow   *)
	s_analog_in.LC0_ACHAN := CHANNELS;
	s_analog_in.LC0_AADDR := buf1;	(* konieczna konwersja na  *)
						(*   daleki adres          *)
	s_analog_in.LC0_ALEN := BUFLEN;	(* dlugosc bufora                  *)
	s_analog_in.LC0_ABMAR := 0;
	s_analog_in.LC0_AEMAR := 0;	(* oba marginesy zerowe            *)
					(* calkowita liczba probek         *)
	s_analog_in.LC0_ASTOP.samples := SAMPLES * CHANNELS;
	r.dx := seg(s_analog_in);
	r.di := ofs(s_analog_in);
	intr(LCinterrupt, r);			(* ANALOG_INPUT    *)
	koniec := 0;
	repeat
	  displaybuf1(bufptr, CHANNELS, SAMPLES);
	until ( koniec <> 0 ) ;
	koniec := 0;
	pressanykey;
	FreeMem(buf1, BUFLEN1);

end;

(****************************************************************************)
(* Przeznaczenie:                                                           *)
(*   Obsluga przerwania przychodzacego z modulu po zmierzeniu bloku         *)
(*   probek - praca asynchroniczna z przerwaniem na koniec bloku            *)
(* Sposob:                                                                  *)
(* Parametry:                                                               *)
(* Wartosc:                                                                 *)
(* Uwagi:                                                                   *)
(****************************************************************************)
{$S-}
procedure a1_interrupt_service;
begin
	if(sstat AND LC0_IS_END_ADC <> 0) then
	  begin
          	  r.dx := seg(s_analog_in);
	          r.di := ofs(s_analog_in);
                  intr(LCinterrupt, r);		(* ANALOG_INPUT             	    *)
					(* funkcja MUSI !! byc wywolana w   *)
					(* obsludze przerwania z modulu     *)
	          koniec := 1;
	          if (bufnum = 1) then
		     bufnum := 2
	          else
		     bufnum := 1;
	  end;
end;
{$S+}
(***************************************************************************)
(* Przeznaczenie:                                                          *)
(*   Wykonanie asynchronicznej transmisji blokowej do dwu buforow.         *)
(* Sposob:                                                                 *)
(*   Przez wykonanie funkcji ANALOG_INPUT.                                 *)
(* Parametry:                                                              *)
(* Wartosc:                                                                *)
(* Uwagi:                                                                  *)
(*   Funkcja wywolujaca musi ustawic okres probkowania i parametry warunku *)
(*   startu.                                                               *)
(***************************************************************************)
procedure asynchrtransmission1;
var
	seg_, last: word;
	Buffer_to_free: pointer;
	bufptr: buftypeptr;
	bufptr2: buftypeptr;
	CurrBuf: buftypeptr;
	CalcBuf: buftypeptr;

begin

	sstat := 0;
	GetMem(buf1, BUFLEN1);
	if(buf1 = nil) then
	begin
	  writeln('BLAD - brak pamieci');
	  Halt(0);
	end;
	bufptr := buftypeptr(buf1);

	GetMem(buf2, BUFLEN1);
	if(buf2 = nil) then
	begin
           FreeMem(buf1, BUFLEN1);
	   writeln('BLAD - brak pamieci');
           Halt(0);
        end;
	bufptr2 := buftypeptr(buf2);

			(* wyinstalowanie obslugi przerwania z modulu      *)
	s_interr.LC0_SMODULE := 1;
	s_interr.LC0_SPROC := nil;
	s_interr.LC0_SSTAT := @sstat;
	r.dx := seg(s_interr);
	r.di := ofs(s_interr);
	r.ds := seg(sstat);
	intr(LCinterrupt, r);                 	(* INTERRUPT_SERVICE *)
			(* zainstalowanie obslugi przerwania z modulu      *)
	s_interr.LC0_SPROC := @a1_interrupt_service;
	r.dx := seg(s_interr);
	r.di := ofs(s_interr);
	r.ds := seg(sstat);
	intr(LCinterrupt, r);			(* INTERRUPT_SERVICE *)

	FillChar(buf1^, SAMPLES * CHANNELS, 0);	(* wstepne zerowanie bufora *)
	FillChar(buf2^, SAMPLES * CHANNELS, 0);

	repeat
		if (bufnum = 1) then
		 begin
			CurrBuf := bufptr;
			CalcBuf := bufptr2;
			FillChar(buf1^, SAMPLES * CHANNELS, 0);
			s_analog_in.LC0_AADDR := buf1;	(* konieczna konwersja na  *)
								(*   daleki adres          *)
		 end
		else
		 begin
			CurrBuf := bufptr2;
			CalcBuf := bufptr;
			FillChar(buf2^, SAMPLES * CHANNELS, 0);
			s_analog_in.LC0_AADDR := buf2;	(* konieczna konwersja na  *)
								(*   daleki adres          *)
		 end;
                s_analog_in.LC0_APER := 500000;
		clrscr;
		gotoxy(1, 1);
		writeln
		(
		'Pomiar blokowy ,' ,CHANNELS, ' kanalow ',SAMPLES, ' probek, okres probkowania [s]: ', s_analog_in.LC0_APER DIV 10
		);
		writeln
		(
		'---- KONIEC (po zapelnieniu bufora) - dowolny klawisz ----'
		);
		writeln;
		writeln;
		writeln
		(
		'Pomiar do bufora nr ', bufnum
		);

		s_analog_in.LC0_AMODULE := modulenum;
		s_analog_in.LC0_ANUM := 1;
		s_analog_in.LC0_AMODE := LC0_MOD_START OR
						LC0_MOD_NEW_PAR OR
						LC0_MOD_INTR OR
						LC0_MOD_BLOCK;
				(* stop po zmierzeniu okreslonej liczby probek     *)
		s_analog_in.LC0_ASTST := LC0_ZSAMPLES;
					(* praca wielokanalowa, CHANNELS kanalow   *)
		s_analog_in.LC0_ACHAN := CHANNELS;
		s_analog_in.LC0_ALEN := SAMPLES * CHANNELS;	(* dlugosc bufora                  *)
		s_analog_in.LC0_ABMAR := 0;
		s_analog_in.LC0_AEMAR := 0;	(* oba marginesy zerowe            *)
						(* calkowita liczba probek         *)
		s_analog_in.LC0_ASTOP.samples := SAMPLES * CHANNELS;
		r.dx := seg(s_analog_in);
		r.di := ofs(s_analog_in);
		intr(LCinterrupt, r);			(* ANALOG_INPUT    *)
		koniec := 0;
		repeat
		  performcalc(CalcBuf, CHANNELS, SAMPLES);
		until ( koniec <> 0 ) ;
		koniec := 0;
	until KeyPressed;
	pressanykey;
	FreeMem(buf1, BUFLEN1);
	FreeMem(buf2, BUFLEN1);

end;

(****************************************************************************)
(* Przeznaczenie:                                                           *)
(*   Obsluga przerwania przychodzacego z modulu po zmierzeniu bloku         *)
(*   probek - praca asynchroniczna z buforem cyklicznym z przerwaniem na    *)
(*   koniec bloku                                                           *)
(* Sposob:                                                                  *)
(* Parametry:                                                               *)
(* Wartosc:                                                                 *)
(* Uwagi:                                                                   *)
(****************************************************************************)
const 	CTONE =	2500;	(* parametry sygnalu informujacego o koncu bloku    *)
const	CDUR = 30;
{$S-}
procedure c_interrupt_service;
var
	a, i : integer;
begin
	if(sstat AND LC0_IS_END_ADC <> 0) then
	  begin
	  r.dx := seg(s_analog_in);
	  r.di := ofs(s_analog_in);
	  FillChar(buf1^, BUFLEN, 0);
	  intr(LCinterrupt, r);
	  end;
end;

{$S+}

(***************************************************************************)
(* Przeznaczenie:                                                          *)
(*   Cykliczne wykonywanie asynchronicznej transmisji blokowej.            *)
(* Sposob:                                                                 *)
(*   Przez wykonanie funkcji ANALOG_INPUT.                                 *)
(* Parametry:                                                              *)
(*   start - typ warunku startu                                            *)
(* Wartosc:                                                                *)
(* Uwagi:                                                                  *)
(*   Funkcja wywolujaca musi ustawic okres probkowania i parametry warunku *)
(*   startu.                                                               *)
(***************************************************************************)
procedure cycletransmission;
var
	seg_, last: word;
	Buffer_to_free: pointer;
	bufptr: buftypeptr;

begin

 GetMem(buf2, BUFLEN1);
 if(buf2 = nil) then
    begin
      writeln('BLAD - brak pamieci');
      Halt(0);
    end;
 seg_ := seg(buf2^) + 1;
 last := seg_ + BUFLEN1 div 16;
 if((seg_ AND $F000) <> (last AND $F000)) then
     buf1 := Ptr((last AND $F000), 0);

 sstat := 0;
 bufptr := buftypeptr(buf1);

	s_interr.LC0_SMODULE := 1;
	s_interr.LC0_SSTAT := @sstat;
	s_interr.LC0_SPROC := nil;
	r.dx := seg(s_interr);
	r.di := ofs(s_interr);
	r.ds := seg(sstat);
	intr(LCinterrupt, r);

	s_interr.LC0_SPROC := @c_interrupt_service;
	r.dx := seg(s_interr);
	r.di := ofs(s_interr);
	r.ds := seg(sstat);
	intr(LCinterrupt, r);



	FillChar(buf1^, BUFLEN, 0);
	s_analog_in.LC0_APER := 2500000;
	writeln
	(
	'Pomiar blokowy ,' ,CHANNELS, ' kanalow ',SAMPLES, ' probek, okres probkowania [s]: ', s_analog_in.LC0_APER DIV 10
	);
	writeln
	(
	'---- praca z buforem cyklicznym; KONIEC - Ctrl-Break ----'
	);
	s_analog_in.LC0_AMODULE := modulenum;
	s_analog_in.LC0_ANUM := 1;
	s_analog_in.LC0_AMODE := LC0_MOD_START OR
			      LC0_MOD_NEW_PAR OR
			      LC0_MOD_INTR OR
			      LC0_MOD_BLOCK OR
			      LC0_MOD_CYCL;
			(* stop po zmierzeniu okreslonej liczby probek     *)
	s_analog_in.LC0_ASTST := LC0_ZSAMPLES;
				(* praca wielokanalowa, CHANNELS kanalow   *)
	s_analog_in.LC0_ACHAN := CHANNELS;
	s_analog_in.LC0_AADDR := buf1;	(* konieczna konwersja na  *)
						(*   daleki adres          *)
	s_analog_in.LC0_ALEN := BUFLEN;	(* dlugosc bufora                  *)
	s_analog_in.LC0_ABMAR := 0;
	s_analog_in.LC0_AEMAR := 0;	(* oba marginesy zerowe            *)
					(* calkowita liczba probek         *)
	s_analog_in.LC0_ASTOP.samples := SAMPLES * CHANNELS;
	r.dx := seg(s_analog_in);
	r.di := ofs(s_analog_in);
	intr(LCinterrupt, r);			(* ANALOG_INPUT    *)
	koniec := 0;
	repeat
	  displaybuf1(bufptr, CHANNELS, SAMPLES);
	until ( koniec <> 0 ) ;
	koniec := 0;
	pressanykey;
	FreeMem(buf2, BUFLEN1);

end;

(***************************************************************************)
(* Przeznaczenie:                                                          *)
(*   Wyslanie pojedynczej wartosci na przetwornik CA.                      *)
(* Sposob:                                                                 *)
(*   Przez wykonanie funkcji ANALOG_OUTPUT.                                *)
(* Parametry:                                                              *)
(* Wartosc:                                                                *)
(* Uwagi:                                                                  *)
(***************************************************************************)
procedure singlewrite;
var
         val:  Integer;			(* wartosc do wyslania             *)
begin
	writeln(
	'---------- Wyslanie pojedynczej wartosci na przetwornik CA'
	);
	if(s_module.LC0_MIDA = 0) then
	begin
		writeln;
		writeln(
		'---------- Karta nie posiada zainstalowanych przetwornikow CA'
		);
		writeln;
		pressanykey;
		Exit;
	end;

	val := 4095;
	s_analog_out.LC0_NMODULE := modulenum;
	s_analog_out.LC0_NNUM := 1;
	s_analog_out.LC0_NMODE := LC0_MOD_START or
				 LC0_MOD_NEW_PAR or
				 LC0_MOD_SYNCHR;
						(* praca jednokanalowa     *)
	s_analog_out.LC0_NCHAN := $80 or $01;
	s_analog_out.LC0_NADDR.base_memory := @val;
					(*   daleki adres                  *)
	s_analog_out.LC0_NLEN := 1;		(* dlugosc bufora (tylko   *)
						(*   na jedna wartosc)     *)
	s_analog_out.LC0_NSTST := LC0_SIMMED + LC0_ZSAMPLES;
						(* praca jednokanalowa     *)
	s_analog_out.LC0_NSTOP.samples := 1;

	r.dx := seg(s_analog_out);
	r.di := ofs(s_analog_out);
	intr(LCinterrupt, r);			(* ANALOG_OUTPUT   *)

	pressanykey;

end;

(***************************************************************************)
(* Przeznaczenie:                                                          *)
(*   Wyslanie przebiegu sinusoidalnego na przetwornik CA.                  *)
(* Sposob:                                                                 *)
(*   Przez wykonanie funkcji ANALOG_OUTPUT.                                *)
(* Parametry:                                                              *)
(* Wartosc:                                                                *)
(* Uwagi:                                                                  *)
(***************************************************************************)
procedure sinewrite;
var
	 DAC_BUFLEN:	word;
         buf: buftype;
	 k, LowWave, UpWave, PerWave, Period:    Real;
	 i, j, range: Integer;
	 c: char;
begin
	if(s_module.LC0_MIDA = 0) then
	begin
		writeln(
		'---------- Wyslanie na przetwornik CA przebiegu sinusoidalnego'
		);
		writeln;
		writeln(
		'---------- Karta nie posiada zainstalowanych przetwornikow CA'
		);
		writeln;
		pressanykey;
		Exit;
	end;

        DAC_BUFLEN := 320;

						(* spytanie o konfiguracje toru c/a*)
	s_info.LC0_GMODULE := modulenum;
	s_info.LC0_GTYPE   := LC0_AOUTPUT;
	s_info.LC0_GNUM    := 1;

	r.dx := Seg(s_info);
	r.di := Ofs(s_info);
	intr(LCinterrupt, r);		(* GET_INFO        *)

        (* obliczamy zdolnosc rozdzielcza przetwornika *)
	range := (s_info.LC0_GMAXV - s_info.LC0_GMINV) DIV 10;
        k := 2;
        for i := 2 to s_info.LC0_GRES do
            k := k * 2;
	k := (k - 1) / range;
					(* PARAMETRY PRZEBIEGU             *)
	LowWave := s_info.LC0_GMINV/10;	 (* dolna wartosc przebiegu [V]     *)
	UpWave  := s_info.LC0_GMAXV/10;  (* gorna wartosc przebiegu [V]     *)
	PerWave := 10.0;                 (* okres przebiegu [ms]            *)
	Period  := 0.1;          			 (* okres wysylania [ms]            *)

	writeln(
	'---------- Wyslanie na przetwornik CA przebiegu sinusoidalnego'
	);
	writeln;
	writeln(
	'---------- Wysylanie mozna przerwac Ctrl-Break'
	);
	writeln;

        j := Round(PerWave/Period);
	for i := 1 to 320 do
                 buf[i] := Round(((UpWave - LowWave) / 2 * Sin(i * 2 * PI / j) + range / 2) * k);

	s_analog_out.LC0_NMODULE := modulenum;
	s_analog_out.LC0_NNUM := 1;
	s_analog_out.LC0_NMODE := LC0_MOD_START or
					LC0_MOD_NEW_PAR or
					LC0_MOD_SYNCHR or
					LC0_MOD_BLOCK or
					LC0_MOD_CYCL;
			(* praca cykliczna, stop po wcisnieciu Ctrl-Break  *)
	s_analog_out.LC0_NSTST := LC0_SIMMED + LC0_ZSAMPLES;
						(* praca jednokanalowa     *)
	s_analog_out.LC0_NCHAN := $80 or $01;
	s_analog_out.LC0_NADDR.base_memory := @buf;
					(*   daleki adres                  *)
	s_analog_out.LC0_NLEN  := DAC_BUFLEN;	(* dlugosc bufora          *)
					(* calkowita liczba probek         *)
	s_analog_out.LC0_NSTOP.samples := j;
	s_analog_out.LC0_NPER  := 1000;
	r.dx := seg(s_analog_out);
	r.di := ofs(s_analog_out);
	intr(LCinterrupt, r);			(* ANALOG_OUTPUT   *)

end;

(***************************************************************************)
(* Przeznaczenie:                                                          *)
(*   Zakonczenie pracy programu.                                           *)
(* Sposob:                                                                 *)
(*   Rozstanie sie z driver'em za pomoca funkcji LEAVE_DRIVER.             *)
(***************************************************************************)
procedure quit;
begin
    r.dx := seg(s_leave);
    r.di := ofs(s_leave);
    intr(LCinterrupt, r);				(* LEAVE_DRIVER    *)

    writeln('Dziekuje, to wszystko!');
end;



begin
	initprogram;		(* inicjalizacja programu                  *)
	checkdrivers;		(* rozpoznanie zainstalowanego driver'a    *)
	askdriver;		(* odpytanie driver'a o konfiguracje       *)
	displayconfiguration;	(* wyswietlenie konfiguracji modulu i      *)
				(* toru a/c                                *)
	installbreak;		(* zainstalowanie procedury obslugi        *)
				(* przerwania generowanego przez Ctrl-Break*)
	pressanykey;		(* czekanie na operatora                   *)

	blocktransmission;	(* wykonanie transmisji blokowej +         *)
				(* wyswietlenie zmierzonych wartosci       *)
	failblocktransmission;	(* wykonanie blednej transmisji blokowej   *)
	interruptedbefore;	(* wykonanie transmisji blokowej przerwanej*)
				(* przez operatora przed startem           *)
	interruptedafter;	(* wykonanie transmisji blokowej przerwanej*)
				(* przez operatora po starcie              *)
	singletransmission;	(* wykonanie pomiaru w trybie pojedynczym  *)

	asynchrtransmission;	(* wykonanie asynchronicznej transmisji blokowej + *)
				(*  wyswietlenie zmierzonych wartosci *)

	asynchrtransmission1;	(* wykonanie asynchronicznej transmisji blokowej *)
				(*  do dwu buforow +                  *)
				(*  wyswietlenie zmierzonych wartosci *)


	cycletransmission;	(* pomiar blokowy do bufora cyklicznego + *)
				(*  wyswietlenie zmierzonych wartosci *)

	singlewrite;	        (* wyslanie pojedynczej wartosci na wyjscie*)

	sinewrite;	        (* wyslanie sinusoidy blokowo na wyjscie   *)

	quit;			(* zakonczenie programu                    *)
end.
